package particles.v1;

import java.awt.Color;
import java.awt.Graphics;
import java.util.Random;

public class Particle implements Runnable
{

    // panneau affichant cette particule
    private ParticlePanel particlePanel;
    
    // position de la particule (initialement : centre du panneau)
    private double x, y;
    
    // taille (largeur et hauteur) de la particule (initialement : 10)
    private int size = 10;
    
    // couleur de la particule (aléatoire)
    private Color color;
    
    // vitesse initiale de la particule (aléatoire, entre -1 et +1)
    private double vx, vy;

    /**
     * constructeur
     * @param particlePanel panneau auquel cette particule est associée
     */
    public Particle(ParticlePanel particlePanel)
    {
		this.particlePanel = particlePanel; 
		this.x= particlePanel.getWidth()/2;
		this.vx = Math.random()*2-1;
		this.vy = Math.random()*2-1;
		// changement de taille this.size = min + (int)(Math.random()*max); 
		this.y= particlePanel.getHeight()/2;
		Random R = new Random();
		int r = R.nextInt(256);
		int v = R.nextInt(256);
		int b = R.nextInt(256);
		this.color = new Color(r,v,b);
    }

    /**
     * dessine la particule sur le contexte graphique fourni
     * Cette méthode sera appelée par la méthode paint d'un ParticlePanel
     * @param g contexte graphique
     */
    public void draw(Graphics g)
    {
        g.setColor(this.color);
        g.fillOval((int)this.x, (int)this.y, this.size, this.size);
    }

    /**
     * met à jour la position et la taille de la particule
     */
    public void update()
    {
        this.x = vx * this.x;
        this.y = vy * this.y;
        this.size = this.size * 2;
    }

    /**
     * retourne la visibilité de la particule (selon sa position, sa taille et celle du panneau particlePanel)
     * @return true si la particule est visible, false sinon
     */
    public boolean isVisible()
    {
        
    	return true;
    }

    /**
     * animation de la particule :
     *
     * Tant que la particule est visible
     *    - mettre à jour la particule (position et taille)
     *    - "demander" au panneau de se redessiner ("repaint")
     *    - "dormir" pendant 40 ms
     *
     * Lorsque la particule n'est plus visible :
     *    - retirer la particule de la liste du panneau
     *    - ajouter une nouvelle particule au panneau
     */
    public void run()
    {
        while(this.isVisible())
        {
        	this.update();
        	this.particlePanel.repaint();
        	try
        	{
				this.wait(40);
			}
        	catch(InterruptedException e)
        	{
				System.out.println("err du wait de l'animation du run :" + e);
			}
        }
        this.particlePanel.removeParticle(this);
        this.particlePanel.addParticle(this);
    }
}
