package particles.v1;

import java.awt.Color;
import java.awt.Graphics;
import java.util.ArrayList;
import java.util.concurrent.Executors;
import java.util.concurrent.ThreadPoolExecutor;

import javax.swing.JPanel;

/**
 * panneau affichant des particules
 */
public class ParticlePanel extends JPanel {

    // liste des particules en cours d'animation
    private final ArrayList<Particle> particles = new ArrayList<Particle>();
    
    // pool à utiliser pour l'animation des particules
    private ThreadPoolExecutor pool = (ThreadPoolExecutor) Executors.newCachedThreadPool();

    /**
     * ajoute une particule à la liste du panneau et lance l'animation de la particule (à l'aide du pool de threads).
     * @param p particule à ajouter
     */
    public void addParticle(Particle p)
    {
        this.particles.add(p);
        this.pool.execute(new Runnable()
        {
			public void run()
			{
			}
		});
    }

    /**
     * retire une particule de la liste du panneau
     * @param p particule à retirer
     */
    public void removeParticle(Particle p)
    {
    	this.particles.remove(p);
    }

    /**
     * retourne le nombre de particules du panneau
     * @return nombre de particules
     */
    public int getParticleCount()
    {
        return this.particles.size();
    }

    /**
     * efface le panneau (dessin d'un rectangle noir) et dessine les particules de la liste (par appel de leur méthode draw).
     * Cette méthode est appelée automatiquement par le thread de gestion des événements
     * chaque fois que le panneau doit être redessiné.
     * @param g contexte graphique permettant de dessiner sur l'espace occupé par le panneau
     */
    @Override
    public void paint(Graphics g)
    {
    	g.setColor(Color.BLACK);
    	for(Particle particle : this.particles)
    	{
 				particle.draw(g);
    	}
    }
}
