package particles.v2;

import java.awt.Color;
import java.awt.Graphics;
import java.util.Random;

public class Particle implements Runnable {

    // panneau affichant cette particule
    private ParticlePanel particlePanel;
    
    // position de la particule (initialement : centre du panneau)
    private double x, y;
    
    // taille (largeur et hauteur) de la particule (initialement : 10)
    private int size = 10;
    
    // couleur de la particule (alÃ©atoire)
    private Color color;
    
    // vitesse initiale de la particule (alÃ©atoire, entre -1 et +1)
    private double vx, vy;

    /**
     * constructeur
     * @param particlePanel panneau auquel cette particule est associÃ©e
     */
    public Particle(ParticlePanel particlePanel) {
    	//int min=10 , max =50 ;
		this.particlePanel = particlePanel; 
		this.x= particlePanel.getWidth()/2;
		this.vx = Math.random()*2-1;
		this.vy = Math.random()*2-1;
		// changement de taille this.size = min + (int)(Math.random()*max); 
		this.y= particlePanel.getHeight()/2;
		Random R = new Random();
		int r = R.nextInt(256);
		int v = R.nextInt(256);
		int b = R.nextInt(256);
		this.color = new Color(r,v,b);
    }

    /**
     * dessine la particule sur le contexte graphique fourni
     * Cette mÃ©thode sera appelÃ©e par la mÃ©thode paint d'un ParticlePanel
     * @param g contexte graphique
     */
    public synchronized void draw(Graphics g) {
    	
		g.setColor(this.color);
		//g.fill3DRect((int)this.x*2, (int)this.y, this.size, this.size,true);
		g.fillOval((int)this.x, (int)this.y, this.size, this.size);
		// test synchronized 
		/*
		Color white = new Color(255,255,255);
		g.setColor(white);
		g.fillOval((int)this.x, (int)this.y, this.size, this.size);*/
		

    }

    /**
     * met Ã  jour la position et la taille de la particule
     */
    public void update() {
    	synchronized (this)
		{
    		this.x += this.vx  * this.size;
     	    this.y += this.vy  * this.size;
        	for ( Particle part : this.particlePanel.getList())

        		collisionTest(part);
		}
  	    this.particlePanel.repaint();
    	

    }

    /**
     * retourne la visibilitÃ© de la particule (selon sa position, sa taille et celle du panneau particlePanel)
     * @return true si la particule est visible, false sinon
     */
    public boolean isVisible() {
    	/*int width = this.particlePanel.getWidth();
    	int heigth = this.particlePanel.getHeight();
    	boolean retour = false;
        if(this.vx < 0 && this.vy < 0)
        {	
        	retour = (this.x+this.size < 0 || this.y+this.size < 0);
        }
        else if(this.vx <0 && this.vy > 0)
        {	
        	retour = (this.x+size <0 || y > heigth);
        }
        else if(this.vx > 0 && this.vy< 0)
        {
        	retour = (this.x > width || this.y+this.size < 0);
        }
        else if(this.vx>0 && this.vy>0 )
        {
        	retour = (this.x > width || this.y > heigth );
        }
        
        return !retour;*/
    	return true;
    }

    /**
     * animation de la particule :
     *
     * Tant que la particule est visible
     *    - mettre Ã  jour la particule (position et taille)
     *    - "demander" au panneau de se redessiner ("repaint")
     *    - "dormir" pendant 40 ms
     *
     * Lorsque la particule n'est plus visible :
     *    - retirer la particule de la liste du panneau
     *    - ajouter une nouvelle particule au panneau
     */
   /* public void run() {
        while(this.isVisible())
        {
     	   this.update();
     	   this.particlePanel.repaint();
     	   try
 		{
 			Thread.sleep(40);
 		} catch (InterruptedException e)
 		{
 			
 			e.printStackTrace();
 		}
     	   
        }
     	  this.particlePanel.removeParticle(this);
    	  this.particlePanel.addParticle(new Particle(this.particlePanel));     	  
        
    }*/
    

    public void isCollidingH()
    {   	
    	
    	int width = this.particlePanel.getWidth();
    	int heigth = this.particlePanel.getHeight();  	
    	
    	if( (x<0 || y+size > heigth) ||
    			(x<0 || y<0) ||  //hg
    			(x+size > width || y<0)  )//hd
    	this.vy *=-1;
    }
    public void  isCollidingG()
    {   	
    	
    	int width = this.particlePanel.getWidth();
    	int heigth = this.particlePanel.getHeight();  	
    	
    	if( (x<0 || y+size > heigth) ||  //bg
    			(x > width || y+size > heigth)   //bd
    			 )
    	this.vx *=-1;
    }
    /**
     * effectue un test de collision entre la particule courante et
     * la particule p. S'il y a collision, les vitesses des particules sont
     * recalculées en fonction des vitesses initiales.
     * @param p particule à tester
     */
    private void collisionTest(Particle p) {
        if (p == this) {
            return;
        }
        double nx = x - p.x;
        double ny = y - p.y;
        double d = Math.sqrt(nx * nx + ny * ny);
        if (d <= size) {
            nx = nx * (size - d) / d;
            ny = ny * (size - d) / d;
            x += nx * 0.5;
            y += ny * 0.5;
            p.x -= nx * 0.5;
            p.y -= ny * 0.5;
            d = Math.sqrt(nx * nx + ny * ny);
            nx /= d;
            ny /= d;
            double vn = (vx - p.vx) * nx + (vy - p.vy) * ny;
            if (vn > 0.0) {
                return;
            }
            vn *= -0.9;
            nx *= vn;
            ny *= vn;
            vx += nx;
            vy += ny;
            p.vx -= nx;
            p.vy -= ny;
        }
    }
    public void run() {
        while(this.isVisible())
        {
     	   this.update();
     	   this.isCollidingG();
     	   this.isCollidingH();
    	  /* if(this.isCollidingH())
    	   {
    		   //this.vx *=-1;
    		   this.vy *=-1;
    	   }
    	   if (this.isCollidingG())
    	   {
    		   this.vx *=-1;
    		  // this.vy *=-1;
    	   }*/
     	   //this.particlePanel.repaint();
    	   try
    	   {
    		   	Thread.sleep(40);
    	   } catch (InterruptedException e)
    	   {
 			
    		   e.printStackTrace();
    	   }
     	   
        }
     	this.particlePanel.removeParticle(this);
    	this.particlePanel.addParticle(new Particle(this.particlePanel));     	  
        
    }
}
