package particles.v2;

import java.awt.Color;
import java.awt.Graphics;
import java.util.ArrayList;
import java.util.concurrent.Executors;
import java.util.concurrent.ThreadPoolExecutor;

import javax.swing.JPanel;

/**
 * panneau affichant des particules
 */
public class ParticlePanel extends JPanel {

    // liste des particules en cours d'animation
    private final ArrayList<Particle> particles = new ArrayList<Particle>();
    
    // pool à utiliser pour l'animation des particules
    private ThreadPoolExecutor pool = (ThreadPoolExecutor) Executors.newCachedThreadPool();

    /**
     * ajoute une particule à la liste du panneau et lance l'animation de la particule (à l'aide du pool de threads).
     * @param p particule à ajouter
     */
    public void addParticle(Particle p) {
    	synchronized (particles)
		{
	        this.particles.add(p);
	        this.pool.submit(p);
		}
        
    }

    /**
     * retire une particule de la liste du panneau
     * @param p particule à retirer
     */
    public void removeParticle(Particle p) {
    	synchronized (particles)
		{
    		this.particles.remove(p);
    		System.out.println("Particule supprime");
		}
       }

    /**
     * retourne le nombre de particules du panneau
     * @return nombre de particules
     */
    public int getParticleCount() {
    	synchronized (particles)
		{
    		return this.particles.size();
		}	
    }
    public ArrayList<Particle> getList(){
    	
    	return this.particles;
    }

    /**
     * efface le panneau (dessin d'un rectangle noir) et dessine les particules de la liste (par appel de leur méthode draw).
     * Cette méthode est appelée automatiquement par le thread de gestion des événements
     * chaque fois que le panneau doit être redessiné.
     * @param g contexte graphique permettant de dessiner sur l'espace occupé par le panneau
     */
    @Override
    public void paint(Graphics g) {
    	Color c = new Color(000, 000, 000);
        g.setColor(c);
        /*test synchronize */

        g.fillRect(0, 0, this.getWidth(), this.getHeight());
        synchronized (particles)
        {
            for(Particle p : this.particles)
            {
         	   p.draw(g);

            }
 		
        }
    }
}